@@ -23,6 +23,12 @@
 #include <sysfs/libsysfs.h>
 #include <regex.h>
 
+/* For globs in /etc/pmount.allow */
+#include <fnmatch.h>
+
+
+#include "realpath.h"
+
 /*************************************************************************
  *
  * Sysfs query functions for determining if a device is removable
@@ -39,6 +45,8 @@
 int
 find_bus_ancestry( struct sysfs_device* dev, char** buses ) {
     char **i;
+    struct sysfs_bus * bus;
+    struct sysfs_device * found;
 
     if( !buses ) {
         debug ( "find_bus_ancestry: no buses to check, fail\n" );
@@ -57,12 +65,54 @@ find_bus_ancestry( struct sysfs_device* dev, char** buses ) {
             return 1;
         }
     }
+    /* Try the hard way */
+    if(find_device_in_buses(dev, buses))
+      return 1;
 
-    debug ( "find_bus_ancestry: device %s (path %s, bus %s) does not match, trying parent\n", 
+    debug ( "find_bus_ancestry: device '%s' (path '%s', bus '%s') "
+	    "does not match, trying parent\n", 
             dev->name, dev->path, dev->bus );
     return find_bus_ancestry( sysfs_get_device_parent( dev ), buses );
 }
 
+/**
+ * Check whether a particular device is found in a list of buses.
+ * @param dev sysfs device
+ * @param buses NULL-terminated array of bus names to scan for
+ * @return 0 if not found, 1 if found
+ */
+
+int
+find_device_in_buses( struct sysfs_device * dev, char** buses) {
+  struct sysfs_bus * bus;
+  struct sysfs_device * found;
+  char ** i;
+
+  for(i = buses; *i; ++i) 
+    {
+      bus = sysfs_open_bus(*i);
+      if(bus)
+	{
+	  debug("find_device_in_buses : "
+		"successfully opened bus '%s' path '%s'\n",
+		bus->name, bus->path);
+	  found = sysfs_get_bus_device(bus,dev->bus_id);
+	  if(found)
+	    {
+	      debug("find_device_in_buses : "
+		    "found '%s' in bus '%s'\n",
+		    dev->name, *i);
+	      sysfs_close_bus(bus);
+	      return 1;		/* We found  it !*/
+	    }
+	  sysfs_close_bus(bus);
+	}
+    }
+  return 0;
+}
+
+
+
 /**
  * Find sysfs node that matches the major and minor device number of the given
  * device. Exit the process immediately on errors.
@@ -362,35 +412,42 @@ device_mounted( const char* device, int expect, char* mntpt )
     return mounted;
 }
 
+/* The silent version of the device_removable function. */
+int device_removable_silent(const char * device)
+{
+  struct sysfs_device *dev;
+  static char* hotplug_buses[] = { "usb", "ieee1394", "mmc", "pcmcia", NULL };
+  int removable;
+  char blockdevpath[PATH_MAX];
+  
+  dev = find_sysfs_device( device, blockdevpath, sizeof( blockdevpath ) );
+  if( !dev ) {
+    debug( "device_removable: could not find a sysfs device for %s\n", device );
+    return 0; 
+  }
+  
+  debug( "device_removable: corresponding block device for %s is %s\n",
+	 device, blockdevpath );
+  
+  /* check whether device has "removable" attribute with value '1' */
+  removable = get_blockdev_attr( blockdevpath, "removable" );
+  
+  /* if not, fall back to bus scanning (regard USB and FireWire as removable) */
+  if( !removable )
+    removable = find_bus_ancestry( dev, hotplug_buses );
+  sysfs_close_device( dev );
+  return removable;
+}
+
 int
 device_removable( const char* device )
 {
-    struct sysfs_device *dev;
-    static char* hotplug_buses[] = { "usb", "ieee1394", "mmc", "pcmcia", NULL };
-    int removable;
-    char blockdevpath[PATH_MAX];
+  int removable = device_removable_silent(device);
 
-    dev = find_sysfs_device( device, blockdevpath, sizeof( blockdevpath ) );
-    if( !dev ) {
-        debug( "device_removable: could not find a sysfs device for %s\n", device );
-        return 0; 
-    }
+  if( !removable )
+    fprintf( stderr, _("Error: device %s is not removable\n"), device );
 
-    debug( "device_removable: corresponding block device for %s is %s\n",
-                device, blockdevpath );
-
-    /* check whether device has "removable" attribute with value '1' */
-    removable = get_blockdev_attr( blockdevpath, "removable" );
-
-    /* if not, fall back to bus scanning (regard USB and FireWire as removable) */
-    if( !removable )
-        removable = find_bus_ancestry( dev, hotplug_buses );
-    sysfs_close_device( dev );
-
-    if( !removable )
-        fprintf( stderr, _("Error: device %s is not removable\n"), device );
-    
-    return removable;
+  return removable;
 }
 
 int
@@ -402,7 +459,7 @@ device_whitelisted( const char* device )
     regex_t re;
     regmatch_t match[3];
     int result;
-    const char* whitelist_regex = "^[[:space:]]*([[:alnum:]/_+.\-]+)[[:space:]]*(#.*)?$";
+    const char* whitelist_regex = "^[[:space:]]*([][:alnum:]/_+.[*?-]+)[[:space:]]*(#.*)?$";
 
     fwl = fopen( WHITELIST, "r" );
     if( !fwl )
@@ -428,7 +485,7 @@ device_whitelisted( const char* device )
            line[match[1].rm_eo] = 0;
            d = line+match[1].rm_so;
            debug( "comparing %s against whitelisted '%s'\n", device, d);
-           if( !strcmp( d, device ) ) {
+	   if( !fnmatch(d, device, FNM_PATHNAME) ) {
                debug( "device_whitlisted(): match, returning 1\n" );
                fclose( fwl );
                return 1;
@@ -489,3 +546,53 @@ void make_lockdir_name( const char* device, char* name, size_t name_size )
     free( devname );
 }
 
+int
+device_valid_silent( const char* device )
+{
+    struct stat st;
+
+    if( stat( device, &st ) ) {
+        return 0;
+    }
+
+    if( !S_ISBLK( st.st_mode ) ) {
+        return 0;
+    }
+
+    return 1;
+}
+
+
+#define PROC_MOUNTS "/proc/mounts"
+#define safe_strcpy(dest, src) snprintf(dest, sizeof(dest), "%s", src);
+
+void print_mounted_removable_devices()
+{
+  FILE* f;
+  struct mntent* ent;
+  char pathbuf[PATH_MAX];
+  /* We need copies, as calls to libsysfs garble the contents of
+     the fields */
+  char name[MEDIA_STRING_SIZE], dir[MEDIA_STRING_SIZE], 
+    type[MEDIA_STRING_SIZE], opts[MEDIA_STRING_SIZE];
+
+  
+  if( !( f = setmntent( PROC_MOUNTS, "r" ) ) ) {
+    fprintf(stderr, _("Error: could not open the %s file: %s"),
+	    PROC_MOUNTS, strerror(errno));
+    exit( 100 );
+  }
+    
+  while( ( ent = getmntent( f ) ) != NULL ) {
+    if(device_valid_silent(ent->mnt_fsname))  {
+      /* We make copies */
+      safe_strcpy(name, ent->mnt_fsname);
+      safe_strcpy(dir, ent->mnt_dir);
+      safe_strcpy(type, ent->mnt_type);
+      safe_strcpy(opts, ent->mnt_opts);
+      if(device_removable_silent(name))
+	printf("%s on %s type %s (%s)\n", name, dir, type, opts);
+    }
+  }
+  endmntent(f);
+}
